/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.business.entity;

import com.baomidou.mybatisplus.annotation.TableName;

import java.io.Serializable;
import java.util.Date;

import io.swagger.annotations.ApiModelProperty;
import org.springblade.core.mp.base.BaseEntity;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 用户意见处理流程表实体类
 *
 * @author BladeX
 * @since 2022-05-25
 */
@Data
@TableName("u_user_opinion_flow")
@EqualsAndHashCode(callSuper = true)
public class UserOpinionFlow extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /**
     * u_user_opinion表主键
     */
	@ApiModelProperty(value = "工单主键")
    private Long userOpinionId;
    /**
     * 是否执行到当前环节，0未执行，1正在执行，2环节结束
     */
	@ApiModelProperty(value = "是否执行到当前环节，0未执行，1正在执行，2环节结束")
    private Integer isCurrent;
    /**
     * 第几次流程，初始为1，触发“不满意并再次提交解决”时在新流程上递增
     */
    private Integer number;
    /**
     * 流程顺序
     */
	@ApiModelProperty(value = "流程顺序")
    private Integer sort;
    /**
     * 分配的处理人
     */
	@ApiModelProperty(value = "分配的处理人")
    private Long manageUser;
    /**
     * 分配的处理人姓名
     */
	@ApiModelProperty(value = "分配的处理人姓名")
    private String manageUserName;
    /**
     * 分配的处理人电话
     */
	@ApiModelProperty(value = "分配的处理人电话")
    private String manageUserPhone;
    /**
     * 评价类型，默认null :	1满意，2不满意并再次提交，3不满意并投诉
     */
	@ApiModelProperty(value = "评价类型：1满意，2不满意并再次提交，3不满意并投诉")
    private Integer evaluation;
	/**
	 * 环节名称
	 */
	@ApiModelProperty(value = "环节名称")
	private String replyName;
	/**
	 * 环节内容
	 */
	@ApiModelProperty(value = "环节内容")
	private String replyContent;

	/**
	 * 环节预计结束时间
	 */
	private String manageTime;

	public UserOpinionFlow(Long userOpinionId, Integer isCurrent, Integer number, Integer sort, String replyName, String replyContent, String manageTime, Long manageUser, String manageUserName, String manageUserPhone){
		this.userOpinionId = userOpinionId;
		this.isCurrent = isCurrent;
		this.number = number;
		this.sort = sort;
		this.manageUser = manageUser;
		this.manageUserName = manageUserName;
		this.manageUserPhone = manageUserPhone;
		this.replyName = replyName;
		this.replyContent = replyContent;
		this.manageTime = manageTime;
		this.setCreateTime(new Date());
	}

    public UserOpinionFlow(){}

}
