/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.business.vo;

import io.swagger.annotations.ApiModelProperty;
import org.springblade.business.entity.UserOpinion;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springblade.business.entity.UserOpinionFile;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 用户意见表视图实体类
 *
 * @author BladeX
 * @since 2022-05-25
 */
@Data
@EqualsAndHashCode(callSuper = true)
public class UserOpinionVO extends UserOpinion {
	private static final long serialVersionUID = 1L;

	private String title;

	/**
	 * 回复量，目前只有一级回复，统计u_user_opinion_comments
	 */
	@ApiModelProperty(value = "回复量")
	private Integer commentsNumber;

	/**
	 * 回显附件URL
	 */
	@ApiModelProperty(value = "回显附件URL")
	private List<String> returnFiles;

	/**
	 * 是否已解决，判断流程第四步是否已经结束，如果最后环节的isCurrent为2则isSolve的值为1，说明已解决
	 */
	@ApiModelProperty(value = "是否已解决, 1为已解决")
	private Integer isSolve = 0;

	/**
	 * 文件，图片，最多3张
	 */
	@ApiModelProperty(value = "上传附件集合")
	private List<MultipartFile> files;

	/**
	 * 用户头像
	 */
	@ApiModelProperty(value = "用户头像")
	private String avatar;

	@ApiModelProperty(value = "当前用户是否对当前工单点赞")
	private Boolean currentUserGood;

}
