/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.dto;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springblade.manager.entity.WbsTree;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;


/**
 * 数据传输对象实体类
 *
 * @author liuyc
 * @since 2022-04-25
 */
@Data
@EqualsAndHashCode(callSuper = true)
public class WbsTreeDTO extends WbsTree {
	private static final long serialVersionUID = 1L;

	/**
	 * wbs库id
	 */
	@NotBlank(message = "wbs库id不能为空")
	@ApiModelProperty(value = "wbs库id",required = true)
	private String wbsId;
	/**
	 * 租户id
	 */
	@NotBlank(message = "租户id不能为空")
	@ApiModelProperty(value = "租户id",required = true)
	private String tenantId;

	/**
	 * 父主键
	 */
	@NotNull(message = "父主键id不能为空")
	@ApiModelProperty(value = "父主键id",required = true)
	private Long parentId;

	/**
	 * 部门类型
	 */
	@ApiModelProperty(value = "节点类型",required = true)
	private Integer deptCategory;

	/**
	 * '1'节点 '2'表单
	 */
	@ApiModelProperty(value = "'1'节点 '2'表单",required = true)
	private Integer type;

	/**
	 * 部门名
	 */
	@NotBlank(message = "名称不能为空")
	@ApiModelProperty(value = "节点名称",required = true)
	private String deptName;
	/**
	 * 部门全称
	 */
	@ApiModelProperty(value = "节点全称",hidden = true)
	private String fullName;

	/**
	 * 表单类型
	 */
	@ApiModelProperty(value = "表单类型",required = true)
	private Integer tableType;

	/**
	 * 排序
	 */
	@ApiModelProperty(value = "排序",required = false)
	private Integer sort;

	/**
	 * 祖级列表
	 */
	@ApiModelProperty(value = "祖级id列表")
	private String ancestors;

	/**
	 * 备注
	 */
	@ApiModelProperty(value = "备注")
	private String remark;

	/**
	 * 唯一编号
	 */
	@ApiModelProperty(value = "唯一编号")
	private String uniqueCode;

	/**
	 * 划分编号
	 */
	@ApiModelProperty(value = "划分编号")
	private String partitionCode;

	/**
	 * 是否是实验节点 '0'否 '1'是
	 */
	@ApiModelProperty(value = "实验节点")
	private Integer isExpernode;

	/**
	 * 是否有混泥土 '0'否 '1'是
	 */
	@ApiModelProperty(value = "混泥土")
	private Integer isConcrete;

	/**
	 * 表单所属方
	 */
	@ApiModelProperty(value = "表单所属方")
	private String tableOwner;

}
