/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.entity;

import com.baomidou.mybatisplus.annotation.TableName;

import java.math.BigDecimal;
import java.time.LocalDateTime;

import io.swagger.annotations.ApiModelProperty;
import org.springblade.core.mp.base.BaseEntity;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 合同段信息表实体类
 *
 * @author liuyc
 * @since 2022-04-14
 */
@Data
@TableName("m_contract_info")
@EqualsAndHashCode(callSuper = true)
public class ContractInfo extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /**
     * 项目id
     */
    @ApiModelProperty(value = "项目id")
    private String pId;

    /**
     * 合同段名称
     */
    @ApiModelProperty(value = "合同段名称")
    private String contractName;
    /**
     * 合同段类型
     */
    @ApiModelProperty(value = "合同段类型")
    private Integer contractType;
    /**
     * 合同段编号
     */
    @ApiModelProperty(value = "合同段编号")
    private String contractNumber;
    /**
     * 业主单位名称
     */
    @ApiModelProperty(value = "业主单位名称")
    private String contractorUnitName;
    /**
     * 施工单位名称
     */
    @ApiModelProperty(value = "施工单位名称")
    private String constructionUnitName;
    /**
     * 监理单位名称
     */
    @ApiModelProperty(value = "监理单位名称")
    private String supervisionUnitName;
    /**
     * 实验室名称
     */
    @ApiModelProperty(value = "实验室名称")
    private String laboratoryName;
    /**
     * 参与方进场时间
     */
    @ApiModelProperty(value = "参与方进场时间")
    private String contractorJoinTime;
    /**
     * 计划开工时间
     */
    @ApiModelProperty(value = "计划开工时间")
    private LocalDateTime planStartTime;
    /**
     * 计划完工时间
     */
    @ApiModelProperty(value = "计划完工时间")
    private LocalDateTime planEndTime;
    /**
     * 实际开工时间
     */
    @ApiModelProperty(value = "实际开工时间")
    private LocalDateTime actualStartTime;
    /**
     * 实际完工时间
     */
    @ApiModelProperty(value = "实际完工时间")
    private LocalDateTime actualEndTime;
    /**
     * 开工令文件
     */
    @ApiModelProperty(value = "开工令文件")
    private String startFileUrl;
    /**
     * 合同段总金额
     */
    @ApiModelProperty(value = "合同段总金额")
    private BigDecimal contractAmount;
    /**
     * 合同段工程位置
     */
    @ApiModelProperty(value = "合同段工程位置")
    private String projectPlace;
    /**
     * 起始桩号
     */
    @ApiModelProperty(value = "起始桩号")
    private String startStation;
    /**
     * 结束桩号
     */
    @ApiModelProperty(value = "结束桩号")
    private String endStation;

    /**
     * 立卷人
     */
    @ApiModelProperty(value = "filer")
    private String filer;

    /**
     * 审核人
     */
    @ApiModelProperty(value = "reviewer")
    private String reviewer;

    /**
     * 档案前缀
     */
    @ApiModelProperty(value = "prefix")
    private String prefix;

    /**
     * 保管期限
     */
    @ApiModelProperty(value = "storagePeriod")
    private Integer storagePeriod;

    /**
     * 保管密级
     */
    @ApiModelProperty(value = "securityLevel")
    private Integer securityLevel;


}
