/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.entity;

import com.baomidou.mybatisplus.annotation.TableName;

import java.io.Serializable;

import io.swagger.annotations.ApiModelProperty;
import org.springblade.core.mp.base.BaseEntity;
import lombok.Data;
import lombok.EqualsAndHashCode;

/**
 * 个人/企业证书信息表实体类
 *
 * @author BladeX
 * @since 2022-06-24
 */
@Data
@TableName("m_sign_pfx_file")
@EqualsAndHashCode(callSuper = true)
public class SignPfxFile extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /**
     * 证书所有者
     */
    @ApiModelProperty("证书所有者")
    private String certificateUserName;
    /**
     * 关联的用户ID
     */
    @ApiModelProperty("关联的用户ID")
    private Long certificateUserId;
    /**
     * 证书ID
     */
    @ApiModelProperty("证书ID")
    private Long certificateId;
    /**
     * 证书编号
     */
    @ApiModelProperty("证书编号")
    private String certificateNumber;
    /**
     * 证书密码
     */
    @ApiModelProperty("证书密码")
    private String certificatePassword;
    /**
     * 证书文件路径
     */
    @ApiModelProperty("证书文件路径")
    private String certificateFileUrl;
    /**
     * 签名文件路径
     */
    @ApiModelProperty("签名文件路径")
    private String signatureFileUrl;
    /**
     * 证书类型，1个人、2企业
     */
    @ApiModelProperty("证书类型，1个人、2企业")
    private Integer certificateType;
    /**
     * 是否注册，0未注册、1已注册
     */
    @ApiModelProperty("是否注册，0未注册、1已注册")
    private Integer isRegister;

    @ApiModelProperty("企业统一社会信用代码")
    private String enterpriseUnifiedCode;

    @ApiModelProperty("规则：项目ID-合同段ID-角色ID,项目ID-合同段ID-角色ID")
    private String projectContractRole;

}
