/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.entity;

import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonIgnore;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springblade.core.mp.base.BaseEntity;


/**
 * 实体类
 *
 * @author liuyc
 * @since 2022-04-25
 */
@Data
@TableName("m_wbs_tree")
@EqualsAndHashCode(callSuper = true)
public class WbsTree extends BaseEntity {

    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @TableId(value = "id")
    private Long id;

    /**
     * wbs库id
     */
    @ApiModelProperty(value = "wbs库id")
    private String wbsId;
    /**
     * 租户id
     */
    @ApiModelProperty(value = "租户id")
    private String tenantId;

    /**
     * 父主键
     */
    @ApiModelProperty(value = "父主键id")
    private Long parentId;

    /**
     * 部门类型
     */
    @ApiModelProperty(value = "节点类型")
    private Integer deptCategory;

    /**
     * '1'节点 '2'表单
     */
    @ApiModelProperty(value = "'1'节点 '2'表单")
    private Integer type;

    /**
     * 部门名
     */
    @ApiModelProperty(value = "节点名称")
    private String deptName;
    /**
     * 部门全称
     */
    @ApiModelProperty(value = "节点全称")
    private String fullName;

    /**
     * 表单类型
     */
    @ApiModelProperty(value = "表单类型")
    private Integer tableType;

    /**
     * 排序
     */
    @ApiModelProperty(value = "排序")
    private Integer sort;

    /**
     * 祖级列表
     */
    @ApiModelProperty(value = "祖级id列表")
    private String ancestors;

    /**
     * 备注
     */
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 唯一编号
     */
    @ApiModelProperty(value = "唯一编号")
    private String uniqueCode;

    /**
     * 划分编号
     */
    @ApiModelProperty(value = "划分编号")
    private String partitionCode;

    /**
     * 是否是实验节点 '0'否 '1'是
     */
    @ApiModelProperty(value = "实验节点")
    private Integer isExpernode;

    /**
     * 是否有混泥土 '0'否 '1'是
     */
    @ApiModelProperty(value = "混泥土")
    private Integer isConcrete;

    /**
     * 表单所属方
     */
    @ApiModelProperty(value = "表单所属方")
    private String tableOwner;

    /**
     * 内业资料类型
     */
    @ApiModelProperty(value = "内业资料类型")
    private Integer majorDataType;

    /**
     * 初始化实体表名
     */
    @ApiModelProperty(value = "初始化实体表名")
    private String initTableName;

    /**
     * 工程根节点id
     */
    @ApiModelProperty(value = "工程根节点id")
    private Long projectNodeId;


    /**
     * 是否关联清表 '0'否 '1'是
     */
    @ApiModelProperty(value = "是否关联清表 '1'否 '2'是")
    private Integer isLinkTable;

    /**
     * 是否存在表单 '0'否 '1'是
     */
    @ApiModelProperty(value = "是否存在表单 '0'否 '1'是")
    private Integer isExistForm;
}
