/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.business.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.business.entity.UserOpinionFile;
import org.springblade.business.vo.UserOpinionFileVO;
import org.springblade.business.service.IUserOpinionFileService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 用户意见附件表 控制器
 *
 * @author BladeX
 * @since 2022-05-25
 */
@RestController
@AllArgsConstructor
@RequestMapping("/userOpinionFile")
@Api(value = "用户意见附件表", tags = "用户意见附件表接口")
public class UserOpinionFileController extends BladeController {

	private final IUserOpinionFileService userOpinionFileService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入userOpinionFile")
	public R<UserOpinionFile> detail(UserOpinionFile userOpinionFile) {
		UserOpinionFile detail = userOpinionFileService.getOne(Condition.getQueryWrapper(userOpinionFile));
		return R.data(detail);
	}

	/**
	 * 分页 用户意见附件表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入userOpinionFile")
	public R<IPage<UserOpinionFile>> list(UserOpinionFile userOpinionFile, Query query) {
		IPage<UserOpinionFile> pages = userOpinionFileService.page(Condition.getPage(query), Condition.getQueryWrapper(userOpinionFile));
		return R.data(pages);
	}

	/**
	 * 自定义分页 用户意见附件表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入userOpinionFile")
	public R<IPage<UserOpinionFileVO>> page(UserOpinionFileVO userOpinionFile, Query query) {
		IPage<UserOpinionFileVO> pages = userOpinionFileService.selectUserOpinionFilePage(Condition.getPage(query), userOpinionFile);
		return R.data(pages);
	}

	/**
	 * 新增 用户意见附件表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入userOpinionFile")
	public R save(@Valid @RequestBody UserOpinionFile userOpinionFile) {
		return R.status(userOpinionFileService.save(userOpinionFile));
	}

	/**
	 * 修改 用户意见附件表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入userOpinionFile")
	public R update(@Valid @RequestBody UserOpinionFile userOpinionFile) {
		return R.status(userOpinionFileService.updateById(userOpinionFile));
	}

	/**
	 * 新增或修改 用户意见附件表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入userOpinionFile")
	public R submit(@Valid @RequestBody UserOpinionFile userOpinionFile) {
		return R.status(userOpinionFileService.saveOrUpdate(userOpinionFile));
	}

	
	/**
	 * 删除 用户意见附件表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(userOpinionFileService.deleteLogic(Func.toLongList(ids)));
	}

	
}
