/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.business.mapper;

import org.springblade.business.entity.ImageClassificationFile;
import org.springblade.business.vo.ImageClassificationFileVO;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import java.util.Date;
import java.util.List;
import org.apache.ibatis.annotations.Param;

/**
 *  Mapper 接口
 *
 * @author BladeX
 * @since 2022-05-24
 */
public interface ImageClassificationFileMapper extends BaseMapper<ImageClassificationFile> {

	/**
	 * 获取当前分类、项目、合同段下的所有文件数量
	 * @param projectId 项目ID
	 * @param contractId 合同段ID
	 * @param classifyId 分类ID
	 * @return 结果
	 */
	Integer queryCurrentClassifyAllFileCount(@Param("projectId") String projectId,@Param("contractId") String contractId,@Param("classifyId") Long classifyId);

	/**
	 * 获取当前分类下当前项目的拍摄时间
	 * @param classifyId 分类ID
	 * @param projectId 项目ID
	 * @return 拍摄时间集合
	 */
	List<Date> selectShootingTimeByClassifyAndProjectId(@Param("classifyId") String classifyId, @Param("projectId") String projectId, @Param("contractId") String contractId);

	/**
	 * 自定义分页
	 *
	 * @param current 分页信息
	 * @param size 分页信息
	 * @param imageClassificationFile 参数
	 * @return 结果
	 */
	List<ImageClassificationFile> selectImageClassificationFilePage(@Param("current")long current, @Param("size")long size, @Param("param") ImageClassificationFileVO imageClassificationFile);

}
