/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.business.service;

import org.springblade.business.entity.InformationQuery;
import org.springblade.business.vo.FileUserVO;
import org.springblade.business.vo.InformationQueryVO;
import org.springblade.core.mp.base.BaseService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.core.secure.BladeUser;

import java.util.List;

/**
 *  服务类
 *
 * @author BladeX
 * @since 2022-06-08
 */
public interface IInformationQueryService extends BaseService<InformationQuery> {

	/**
	 * 保存填报时新增或修改填报资料记录表数据
	 * @param user 当前操作用户
	 * @param wbsId 当前填报节点
	 * @param fileName 文件题名
	 * @param classify 1施工2质检
	 * @param sourceType 1原生2数据化
	 * @param dataId 1原生2数据化
	 */
	void saveOrUpdateInformationQueryData(BladeUser user, Long wbsId, String fileName, Integer classify, Integer sourceType, Long dataId);

	/**
	 * 获取当前合同段下所有的上报批次
	 * @param classify 合同段类型
	 * @param contractId 合同段ID
	 * @return 结果
	 */
	List<Integer> getReportNumberByContractId(Integer classify, String contractId);

	/**
	 * 根据合同段ID获取所有填报人
	 * @param classify 合同段类型
	 * @param contractId 合同段ID
	 * @return 填报人集合
	 */
	List<FileUserVO> queryFileUserByContractId(Integer classify, String contractId);

	/**
	 * 自定义分页
	 */
	IPage<InformationQueryVO> selectInformationQueryPage(IPage<InformationQueryVO> page, InformationQueryVO vo);

}
