/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.manager.entity.ExctabCell;
import org.springblade.manager.vo.ExctabCellVO;
import org.springblade.manager.service.IExctabCellService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * Excel 结构信息表 控制器
 *
 * @author BladeX
 * @since 2022-05-26
 */
@RestController
@AllArgsConstructor
@RequestMapping("/exctabcell")
@Api(value = "Excel 结构信息表", tags = "Excel 结构信息表接口")
public class ExctabCellController extends BladeController {

	private final IExctabCellService exctabCellService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入exctabCell")
	public R<ExctabCell> detail(ExctabCell exctabCell) {
		ExctabCell detail = exctabCellService.getOne(Condition.getQueryWrapper(exctabCell));
		return R.data(detail);
	}

	/**
	 * 分页 Excel 结构信息表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入exctabCell")
	public R<IPage<ExctabCell>> list(ExctabCell exctabCell, Query query) {
		IPage<ExctabCell> pages = exctabCellService.page(Condition.getPage(query), Condition.getQueryWrapper(exctabCell));
		return R.data(pages);
	}

	/**
	 * 自定义分页 Excel 结构信息表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入exctabCell")
	public R<IPage<ExctabCellVO>> page(ExctabCellVO exctabCell, Query query) {
		IPage<ExctabCellVO> pages = exctabCellService.selectExctabCellPage(Condition.getPage(query), exctabCell);
		return R.data(pages);
	}

	/**
	 * 新增 Excel 结构信息表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入exctabCell")
	public R save(@Valid @RequestBody ExctabCell exctabCell) {
		return R.status(exctabCellService.save(exctabCell));
	}


	/**
	 * 修改 Excel 结构信息表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入exctabCell")
	public R update(@Valid @RequestBody ExctabCell exctabCell) {
		return R.status(exctabCellService.updateById(exctabCell));
	}

	/**
	 * 新增或修改 Excel 结构信息表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入exctabCell")
	public R submit(@Valid @RequestBody ExctabCell exctabCell) {
		return R.status(exctabCellService.saveOrUpdate(exctabCell));
	}

	
	/**
	 * 删除 Excel 结构信息表
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		return R.status(exctabCellService.removeByIds(Func.toLongList(ids)));
	}

	
}
