/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;

import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;

import org.springblade.manager.entity.ImageClassificationConfig;
import org.springblade.manager.vo.ImageClassificationConfigVO;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.manager.wrapper.ImageClassificationConfigWrapper;
import org.springblade.manager.service.IImageClassificationConfigService;
import org.springblade.core.boot.ctrl.BladeController;

import java.util.List;

/**
 * 影像分类配置表 控制器
 *
 * @author liuyc
 * @since 2022-05-16
 */
@RestController
@AllArgsConstructor
@RequestMapping("/imageClassificationConfig")
@Api(value = "影像分类配置", tags = "影像分类配置接口")
public class ImageClassificationConfigController extends BladeController {

    private final IImageClassificationConfigService imageClassificationConfigService;

    /**
     * 详情
     */
    @GetMapping("/detail")
    @ApiOperationSupport(order = 1)
    @ApiOperation(value = "详情", notes = "传入imageClassificationConfig")
    public R<ImageClassificationConfigVO> detail(ImageClassificationConfig imageClassificationConfig) {
        ImageClassificationConfig detail = imageClassificationConfigService.getOne(Condition.getQueryWrapper(imageClassificationConfig));
        return R.data(ImageClassificationConfigWrapper.build().entityVO(detail));
    }

    /**
     * 分页 影像分类配置表
     */
    @GetMapping("/list")
    @ApiOperationSupport(order = 2)
    @ApiOperation(value = "分页", notes = "传入imageClassificationConfig")
    public R<IPage<ImageClassificationConfigVO>> list(ImageClassificationConfig imageClassificationConfig, Query query) {
        IPage<ImageClassificationConfig> pages = imageClassificationConfigService.page(Condition.getPage(query), Condition.getQueryWrapper(imageClassificationConfig));
        return R.data(ImageClassificationConfigWrapper.build().pageVO(pages));
    }


    /**
     * 自定义分页 影像分类配置表
     */
    @GetMapping("/page")
    @ApiOperationSupport(order = 3)
    @ApiOperation(value = "分页", notes = "传入imageClassificationConfig")
    public R<IPage<ImageClassificationConfigVO>> page(ImageClassificationConfigVO imageClassificationConfig, Query query) {
        IPage<ImageClassificationConfigVO> pages = imageClassificationConfigService.selectImageClassificationConfigPage(Condition.getPage(query), imageClassificationConfig);
        return R.data(pages);
    }

    /**
     * 新增 影像分类配置表
     */
    @PostMapping("/save")
    @ApiOperationSupport(order = 4)
    @ApiOperation(value = "新增", notes = "传入imageClassificationConfig")
    public R save(@Valid @RequestBody ImageClassificationConfig imageClassificationConfig) {
        return R.status(imageClassificationConfigService.save(imageClassificationConfig));
    }

    /**
     * 修改 影像分类配置表
     */
    @PostMapping("/update")
    @ApiOperationSupport(order = 5)
    @ApiOperation(value = "修改", notes = "传入imageClassificationConfig")
    public R update(@Valid @RequestBody ImageClassificationConfig imageClassificationConfig) {
        return R.status(imageClassificationConfigService.updateById(imageClassificationConfig));
    }

    /**
     * 新增或修改 影像分类配置表
     */
    @PostMapping("/submit")
    @ApiOperationSupport(order = 6)
    @ApiOperation(value = "新增或修改", notes = "传入imageClassificationConfig")
    public R submit(@Valid @RequestBody ImageClassificationConfig imageClassificationConfig) {
        return R.status(imageClassificationConfigService.saveOrUpdate(imageClassificationConfig));
    }


    /**
     * 删除 影像分类配置表
     */
    @PostMapping("/remove")
    @ApiOperationSupport(order = 7)
    @ApiOperation(value = "逻辑删除", notes = "传入ids")
    public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
        return R.status(imageClassificationConfigService.deleteLogic(Func.toLongList(ids)));
    }

    /**
     * 根据所属类型查询
     */
    @GetMapping("/findByType")
    @ApiOperationSupport(order = 8)
    @ApiOperation(value = "根据所属类型查询", notes = "传入type")
    public R<List<ImageClassificationConfig>> findByType(@RequestParam("type") String type) {

        List<ImageClassificationConfig> lists = imageClassificationConfigService.selectByType(type);

        if (lists.size() > 0) {
            return R.data(lists,"查询成功");
        }
        return R.fail("未查询到数据");
    }


}
