/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.manager.service.IWbsFormElementService;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.manager.entity.LinkdataInfo;
import org.springblade.manager.vo.LinkdataInfoVO;
import org.springblade.manager.service.ILinkdataInfoService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 清表解析字段信息 控制器
 *
 * @author BladeX
 * @since 2022-06-17
 */
@RestController
@AllArgsConstructor
@RequestMapping("/linkdatainfo")
@Api(value = "清表解析字段信息", tags = "清表解析字段信息接口")
public class LinkdataInfoController extends BladeController {

	private final ILinkdataInfoService linkdataInfoService;

	private final IWbsFormElementService wbsFormElementService;
	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入linkdataInfo")
	public R<LinkdataInfo> detail(LinkdataInfo linkdataInfo) {
		LinkdataInfo detail = linkdataInfoService.getOne(Condition.getQueryWrapper(linkdataInfo));
		return R.data(detail);
	}


/*
	*//**
	 * 分页 清表解析字段信息
	 *//*
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入linkdataInfo")
	public R<IPage<LinkdataInfo>> list(LinkdataInfo linkdataInfo, Query query) {
		IPage<LinkdataInfo> pages = linkdataInfoService.page(Condition.getPage(query), Condition.getQueryWrapper(linkdataInfo));
		return R.data(pages);
	}

	*//**
	 * 自定义分页 清表解析字段信息
	 *//*
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入linkdataInfo")
	public R<IPage<LinkdataInfoVO>> page(LinkdataInfoVO linkdataInfo, Query query) {
		IPage<LinkdataInfoVO> pages = linkdataInfoService.selectLinkdataInfoPage(Condition.getPage(query), linkdataInfo);
		return R.data(pages);
	}*/

	/**
	 * 新增 清表解析字段信息
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入linkdataInfo")
	public R save(@Valid @RequestBody LinkdataInfo linkdataInfo) {
		return R.status(linkdataInfoService.save(linkdataInfo));
	}

	/**
	 * 修改 清表解析字段信息
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入linkdataInfo")
	public R update(@Valid @RequestBody LinkdataInfo linkdataInfo) {
		return R.status(linkdataInfoService.updateById(linkdataInfo));
	}

	/**
	 * 新增或修改 清表解析字段信息
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入linkdataInfo")
	public R submit(@Valid @RequestBody LinkdataInfo linkdataInfo) {

		return R.status(linkdataInfoService.saveOrUpdate(linkdataInfo));
	}

	
	/**
	 * 删除 清表解析字段信息
	 */
	@PostMapping("/remove")
	@ApiOperationSupport(order = 7)
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		linkdataInfoService.removeByIds(Func.toLongList(ids));
		return R.success("成功！");
	}

	/**
	 *  获取excel字段信息 下拉框
	 */
	@GetMapping("/getColByTabId")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "获取excel字段信息", notes = "传入ids")
	public R getColByTabId(@ApiParam(value = "主键集合", required = true) @RequestParam String tabId) {
		return R.data(linkdataInfoService.getColSelect(tabId));
	}


	/**
	 *  获取元素信息通过tableId 下拉框
	 */
	@GetMapping("/getElementInfoByTabId")
	@ApiOperationSupport(order = 8)
	@ApiOperation(value = "获取excel字段信息", notes = "传入ids")
	public R getElementInfo(@ApiParam(value = "主键集合", required = true) @RequestParam String tabId) {
		return R.data(wbsFormElementService.selectElementListByFid(tabId));
	}

}
