/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import lombok.AllArgsConstructor;
import javax.validation.Valid;

import org.springblade.core.mp.support.Condition;
import org.springblade.core.mp.support.Query;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.Func;
import org.springblade.manager.vo.TextdictDataInfoVO;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.RequestParam;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springblade.manager.entity.TextdictInfo;
import org.springblade.manager.vo.TextdictInfoVO;
import org.springblade.manager.service.ITextdictInfoService;
import org.springblade.core.boot.ctrl.BladeController;

/**
 * 参数信息表 控制器
 *
 * @author BladeX
 * @since 2022-06-17
 */
@RestController
@AllArgsConstructor
@RequestMapping("/textdictinfo")
@Api(value = "参数信息表", tags = "参数信息表接口")
public class TextdictInfoController extends BladeController {

	private final ITextdictInfoService textdictInfoService;

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperationSupport(order = 1)
	@ApiOperation(value = "详情", notes = "传入textdictInfo")
	public R<TextdictInfo> detail(TextdictInfo textdictInfo) {
		TextdictInfo detail = textdictInfoService.getOne(Condition.getQueryWrapper(textdictInfo));
		return R.data(detail);
	}

	/**
	 * 分页 参数信息表
	 */
	@GetMapping("/list")
	@ApiOperationSupport(order = 2)
	@ApiOperation(value = "分页", notes = "传入textdictInfo")
	public R<IPage<TextdictInfo>> list(TextdictInfo textdictInfo, Query query) {
		IPage<TextdictInfo> pages = textdictInfoService.page(Condition.getPage(query), Condition.getQueryWrapper(textdictInfo));
		return R.data(pages);
	}

	/**
	 * 自定义分页 参数信息表
	 */
	@GetMapping("/page")
	@ApiOperationSupport(order = 3)
	@ApiOperation(value = "分页", notes = "传入textdictInfo")
	public R<IPage<TextdictInfoVO>> page(TextdictInfoVO textdictInfo, Query query) {
		IPage<TextdictInfoVO> pages = textdictInfoService.selectTextdictInfoPage(Condition.getPage(query), textdictInfo);
		return R.data(pages);
	}

	/**
	 * 新增 参数信息表
	 */
	@PostMapping("/save")
	@ApiOperationSupport(order = 4)
	@ApiOperation(value = "新增", notes = "传入textdictInfo")
	public R save(@Valid @RequestBody TextdictInfo textdictInfo) {
		return R.status(textdictInfoService.save(textdictInfo));
	}

	/**
	 * 修改 参数信息表
	 */
	@PostMapping("/update")
	@ApiOperationSupport(order = 5)
	@ApiOperation(value = "修改", notes = "传入textdictInfo")
	public R update(@Valid @RequestBody TextdictInfo textdictInfo) {
		return R.status(textdictInfoService.updateById(textdictInfo));
	}

	/**
	 * 新增或修改 参数信息表
	 */
	@PostMapping("/submit")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入textdictInfo")
	public R submit(@Valid @RequestBody TextdictInfo textdictInfo) {
		return R.status(textdictInfoService.saveOrUpdate(textdictInfo));
	}

	
	/**
	 * 删除 参数信息表
	 */
//	@PostMapping("/remove")
//	@ApiOperationSupport(order = 7)
//	@ApiOperation(value = "逻辑删除", notes = "传入ids")
//	public R remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
//		return R.status(textdictInfoService.deleteLogic(Func.toLongList(ids)));
//	}


	/**
	 * 保存字段信息
	 */
	@PostMapping("/saveTextInfo")
	@ApiOperationSupport(order = 6)
	@ApiOperation(value = "新增或修改", notes = "传入textdictInfo")
	public R saveTextInfo(@Valid @RequestBody TextdictDataInfoVO textdictInfo) {
		//

		return R.status(textdictInfoService.saveBatch(textdictInfo.getTextInfo()));
	}

}
