/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.service.impl;

import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.node.ForestNodeMerger;
import org.springblade.core.tool.utils.StringPool;
import org.springblade.manager.entity.ExcelTab;
import org.springblade.manager.vo.ExceTabTreVO;
import org.springblade.manager.vo.ExcelTabVO;
import org.springblade.manager.mapper.ExcelTabMapper;
import org.springblade.manager.service.IExcelTabService;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.manager.vo.ExcelTabWbsTypeVO;
import org.springblade.manager.vo.WbsTreeVO;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.metadata.IPage;

import java.util.List;

/**
 * 清表基础数据表 服务实现类
 *
 * @author BladeX
 * @since 2022-05-18
 */
@Service
public class ExcelTabServiceImpl extends BaseServiceImpl<ExcelTabMapper, ExcelTab> implements IExcelTabService {

	@Override
	public IPage<ExcelTabVO> selectExcelTabPage(IPage<ExcelTabVO> page, ExcelTabVO excelTab) {
		return page.setRecords(baseMapper.selectExcelTabPage(page, excelTab));
	}

	@Override
	public List<ExceTabTreVO> tabLazyTree(String tenantId, Long modeId,Long parentId) {
		if (AuthUtil.isAdministrator()) {
			tenantId = StringPool.EMPTY;
		}
		return ForestNodeMerger.merge(baseMapper.tabLazyTree(tenantId, modeId,parentId));
	}

	@Override
	public List<ExceTabTreVO> tabLazyTreeAll(String tenantId, Long modeId,String name) {
		if (AuthUtil.isAdministrator()) {
			tenantId = StringPool.EMPTY;
		}
		return ForestNodeMerger.merge(baseMapper.tabLazyTreeAll(tenantId, modeId,name));
	}

	@Override
	public List<ExcelTabWbsTypeVO> getWbsTypeList() {
		return baseMapper.getWbsTypeList();
	}

	@Override
	public List<WbsTreeVO> lazyTree(String wbsId, String tenantId, Long parentId) {
		if (AuthUtil.isAdministrator()) {
			tenantId = StringPool.EMPTY;
		}
		return ForestNodeMerger.merge(baseMapper.lazyTree(wbsId, tenantId, parentId));
	}
}
