/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.service.impl;

import org.springblade.manager.entity.ProjectInfo;
import org.springblade.manager.entity.WbsInfo;
import org.springblade.manager.entity.WbsTreePrivate;
import org.springblade.manager.mapper.*;
import org.springblade.manager.vo.ContractlnfoCountVO;
import org.springblade.manager.vo.ProjectInfoVO;
import org.springblade.manager.service.IProjectInfoService;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.manager.vo.ProjectInfoVO2;
import org.springblade.manager.vo.SingPfxManagementVO;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.metadata.IPage;

import javax.annotation.Resource;
import java.util.List;

/**
 * 客户级项目表 服务实现类
 *
 * @author liuyc
 * @since 2022-04-13
 */
@Service
public class ProjectInfoServiceImpl extends BaseServiceImpl<ProjectInfoMapper, ProjectInfo> implements IProjectInfoService {

    @Resource
    private ContractInfoMapper contractInfoMapper;
    @Resource
    private WbsInfoMapper wbsInfoMapper;
    @Resource
    private WbsTreePrivateMapper wbsTreePrivateMapper;

    @Resource
    private SignPfxFileMapper signPfxFileMapper;

    @Override
    public List<SingPfxManagementVO> singPfxManagementPage(SingPfxManagementVO vo) {
        long current = (vo.getCurrent() - 1L) * vo.getSize();
        List<SingPfxManagementVO> result = this.baseMapper.singPfxManagementPage(current, vo.getSize(), vo);

        result.forEach(vos -> {
            //查询个人签章
            vos.setPersonalCount(this.signPfxFileMapper.queryPersonalOrEnterpriseCount(vos.getProjectId(), 1).toString());
            //查询企业章
            vos.setEnterpriseCount(this.signPfxFileMapper.queryPersonalOrEnterpriseCount(vos.getProjectId(), 2).toString());
        });

        return result;
    }

    @Override
    public List<ProjectInfo> selectProjectList(List<String> projectIds) {
        return this.baseMapper.selectProjectList(projectIds);
    }


    @Override
    public IPage<ProjectInfoVO> selectProjectInfoPage(IPage<ProjectInfoVO> page, ProjectInfoVO projectInfo) {
        return page.setRecords(baseMapper.selectProjectInfoPage(page, projectInfo));
    }

    @Override
    public List<ContractlnfoCountVO> selectContractInfoCount() {
        List<ContractlnfoCountVO> contractlnfoCountVO = contractInfoMapper.selectContractInfoCount();
        return contractlnfoCountVO;
    }


    @Override
    public ProjectInfo getOne(Long id) {
        return baseMapper.selectById(id);
    }

    @Override
    public ProjectInfoVO2 getOneByCondition(Long id, Long referenceWbsTemplateId, String referenceWbsTemplateType) {
        ProjectInfoVO2 projectInfoVO2 = new ProjectInfoVO2();
        ProjectInfo projectInfo = baseMapper.selectById(id);
        if ("public".equals(referenceWbsTemplateType) && referenceWbsTemplateId != (-1)) {
            WbsInfo wbsInfo = wbsInfoMapper.selectById(referenceWbsTemplateId);
            projectInfoVO2.setWbsType(wbsInfo.getWbsType());
            projectInfoVO2.setProjectInfo(projectInfo);
        } else if (("private").equals(referenceWbsTemplateType) && referenceWbsTemplateId != (-1)) {
            WbsTreePrivate wbsTreePrivate = wbsTreePrivateMapper.selectByPKeyId(referenceWbsTemplateId);
            projectInfoVO2.setWbsType(Integer.valueOf(wbsTreePrivate.getWbsType()));
            projectInfoVO2.setProjectInfo(projectInfo);
        } else {
            projectInfoVO2.setProjectInfo(projectInfo);
        }
        return projectInfoVO2;
    }


}
