/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.AllArgsConstructor;
import oracle.sql.BfileDBAccess;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.manager.dto.WbsInfoDTO;
import org.springblade.manager.entity.WbsTree;
import org.springblade.manager.mapper.WbsInfoMapper;
import org.springblade.manager.service.IWbsInfoService;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.manager.entity.WbsInfo;
import org.springblade.manager.vo.WbsInfoVO;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * wbs动态库信息 服务实现类
 *
 * @author liuyc
 * @since 2022-04-25
 */
@Service
@AllArgsConstructor
public class WbsInfoServiceImpl extends BaseServiceImpl<WbsInfoMapper, WbsInfo> implements IWbsInfoService {

    private final WbsTreeServiceImpl wbsTreeService;

    @Override
    public IPage<WbsInfo> selectWbsInfoPage(IPage<WbsInfo> page, WbsInfo wbsInfo) {
        return page.setRecords(baseMapper.selectWbsInfoPage(page, wbsInfo));
    }

    @Override
    public List<WbsInfo> findByWbsType(String type) {
        QueryWrapper<WbsInfo> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("wbs_type", type);
        return baseMapper.selectList(queryWrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean submit(WbsInfoDTO wbsInfo) {
        WbsTree wbsTree1 = wbsTreeService.selectGenNodeByWbsId(wbsInfo.getId());
        if (wbsTree1 != null) {
            //修改WbsInfo树信息
            baseMapper.updateById(wbsInfo);
            //修改WbsTree表的根节点
            int row = wbsTreeService.updateByCondition(wbsInfo.getId(), wbsInfo.getWbsName(), wbsInfo.getStatus());
            if (row > 0) {
                return true;
            }
        } else {
            //新增wbs树
            int row = baseMapper.insert(wbsInfo);
            if (row > 0) {
                Long wbsId = wbsInfo.getId();
                String nodeName = wbsInfo.getWbsName();
                //初始化wbs树的根节点
                WbsTree wbsTree = new WbsTree();
                wbsTree.setWbsId(String.valueOf(wbsId));
                wbsTree.setTenantId(AuthUtil.getTenantId());
                wbsTree.setParentId(0L);
                wbsTree.setAncestors("0");
                wbsTree.setDeptCategory(1);
                wbsTree.setDeptName(nodeName);
                wbsTree.setFullName(nodeName);
                wbsTree.setType(1);
                boolean result2 = wbsTreeService.save(wbsTree);
                if (result2) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public WbsInfo findById(Long id) {
        return baseMapper.selectById2(id);
    }

}
