/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.manager.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.AllArgsConstructor;
import net.sourceforge.pinyin4j.PinyinHelper;
import net.sourceforge.pinyin4j.format.HanyuPinyinCaseType;
import net.sourceforge.pinyin4j.format.HanyuPinyinOutputFormat;
import net.sourceforge.pinyin4j.format.HanyuPinyinToneType;
import net.sourceforge.pinyin4j.format.HanyuPinyinVCharType;
import net.sourceforge.pinyin4j.format.exception.BadHanyuPinyinOutputFormatCombination;
import org.apache.commons.lang.StringUtils;
import org.springblade.common.utils.SnowFlakeUtil;
import org.springblade.core.log.exception.ServiceException;
import org.springblade.core.secure.utils.AuthUtil;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.constant.BladeConstant;
import org.springblade.core.tool.node.ForestNodeMerger;
import org.springblade.core.tool.utils.DateUtil;
import org.springblade.core.tool.utils.Func;
import org.springblade.core.tool.utils.StringPool;
import org.springblade.manager.dto.FormElementDTO;
import org.springblade.manager.dto.WbsTreeBatchImportDTO;
import org.springblade.manager.dto.WbsTreeContractDTO;
import org.springblade.manager.dto.WbsTreeDTO2;
import org.springblade.manager.entity.*;
import org.springblade.manager.excel.WbsExcelBatchUtil;
import org.springblade.manager.excel.WbsExcelUtil;
import org.springblade.manager.mapper.*;
import org.springblade.manager.service.IWbsFormElementService;
import org.springblade.manager.service.IWbsTreeService;
import org.springblade.core.mp.base.BaseServiceImpl;
import org.springblade.manager.vo.*;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 服务实现类
 *
 * @author liuyc
 * @since 2022-04-25
 */
@Service
@AllArgsConstructor
public class WbsTreeServiceImpl extends BaseServiceImpl<WbsTreeMapper, WbsTree> implements IWbsTreeService {

    private final WbsTreeMapper wbsTreeMapper;
    private final IWbsFormElementService wbsFormElementService;
    private final WbsInfoMapper wbsInfoMapper;
    private final WbsTreePrivateMapper wbsTreePrivateMapper;
    private final WbsTreePrivateServiceImpl wbsTreePrivateService;
    private final ProjectInfoMapper projectInfoMapper;


    @Override
    public IPage<WbsTreeVO> selectWbsTreePage(IPage<WbsTreeVO> page, WbsTreeVO wbsTree) {
        return page.setRecords(baseMapper.selectWbsTreePage(page, wbsTree));
    }

    @Override
    public List<WbsTreeVO> lazyTree(String wbsId, String tenantId, Long parentId) {
        if (AuthUtil.isAdministrator()) {
            tenantId = StringPool.EMPTY;
        }
        return ForestNodeMerger.merge(baseMapper.lazyTree(wbsId, tenantId, parentId));
    }

    @Override
    public List<WbsTreeVO> tree(String wbsId, String tenantId, String type) {
        return ForestNodeMerger.merge(baseMapper.tree(wbsId, tenantId, type));
    }

    @Override
    public boolean submit(WbsTree dept) {
        if (Func.isEmpty(dept.getParentId())) {
            dept.setTenantId(AuthUtil.getTenantId());
            dept.setParentId(BladeConstant.TOP_PARENT_ID);
            dept.setAncestors(String.valueOf(BladeConstant.TOP_PARENT_ID));
        }
        if (dept.getParentId() > 0) {
            WbsTree parent = getById(dept.getParentId());
            if (Func.toLong(dept.getParentId()) == Func.toLong(dept.getId())) {
                throw new ServiceException("父节点不可选择自身!");
            }
            dept.setTenantId(parent.getTenantId());
            String ancestors = parent.getAncestors() + StringPool.COMMA + dept.getParentId();
            dept.setAncestors(ancestors);
        }
        if (dept.getMajorDataType() == null) {
            dept.setMajorDataType(0);
        }
        dept.setStatus(1);
        dept.setIsDeleted(BladeConstant.DB_NOT_DELETED);
        return saveOrUpdate(dept);
    }

    public boolean submit2(FormElementDTO dept) {
        if (Func.isEmpty(dept.getParentId())) {
            dept.setTenantId(AuthUtil.getTenantId());
            dept.setParentId(BladeConstant.TOP_PARENT_ID);
            dept.setAncestors(String.valueOf(BladeConstant.TOP_PARENT_ID));
        }
        if (dept.getParentId() > 0) {
            WbsTree parent = getById(dept.getParentId());
            if (Func.toLong(dept.getParentId()) == Func.toLong(dept.getId())) {
                throw new ServiceException("父节点不可选择自身!");
            }
            dept.setTenantId(parent.getTenantId());
            String ancestors = parent.getAncestors() + StringPool.COMMA + dept.getParentId();
            dept.setAncestors(ancestors);
            //设置表示该父节点下存在表单
            baseMapper.updateIsExistFormById(parent.getId());
        }
        //设置类型默认值为 '2' 表单
        dept.setType(2);
        dept.setIsDeleted(BladeConstant.DB_NOT_DELETED);
        return saveOrUpdate(dept);

    }

    public boolean submit3(WbsTree dept) {
        if (dept.getParentId() > 0) {
            //设置表示该父节点下存在表单
            baseMapper.updateIsExistFormById(dept.getParentId());
        }
        return save(dept);

    }

    /**
     * wbs元素表、元素批量导入
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean importWbsTreeTableAndForm(MultipartFile excelFile, WbsTree wbsTree) {
        String canonicalPath = null;
        try {
            //解析
            File file = WbsExcelBatchUtil.convert(excelFile);
            canonicalPath = file.getCanonicalPath();
            WbsExcelBatchUtil excelUtil = new WbsExcelBatchUtil();
            ArrayList<Map<String, String>> result = excelUtil.readExcelToObj(canonicalPath);
            List<Map<String, String>> result2 = WbsExcelBatchUtil.deepCopy(result);
            List<Map<String, String>> maps = WbsExcelBatchUtil.removeRepeatMapByKey(result, "表名");
            assert maps != null;
            List<WbsTreeBatchImportDTO> list = new ArrayList<>();
            //初始化
            wbsTree.setTenantId(wbsTree.getTenantId());
            wbsTree.setDeptCategory(1);
            wbsTree.setWbsId(wbsTree.getWbsId());
            wbsTree.setParentId(wbsTree.getId());
            String ancestors = wbsTree.getAncestors() + StringPool.COMMA + wbsTree.getId();
            wbsTree.setAncestors(ancestors);
            wbsTree.setType(2);
            wbsTree.setIsDeleted(BladeConstant.DB_NOT_DELETED);
            maps.stream().forEach(map->{
                String deptName = map.get("表名");
                wbsTree.setDeptName(deptName);
                wbsTree.setTableType(getTableType(map.get("表类型")));
                Long id = SnowFlakeUtil.getId();
                String initTableName = "m_wbs_form_" + DateUtil.time() + "_" + id.toString();
                wbsTree.setInitTableName(initTableName);
                wbsTree.setId(id);
                //新增表单
                boolean b = submit3(wbsTree);
                if (b) {
                    //新增实体表
                    initTable(initTableName);
                    //返参
                    WbsTreeBatchImportDTO wbsTreeBatchImportDTO = new WbsTreeBatchImportDTO();
                    wbsTreeBatchImportDTO.setId(id);
                    wbsTreeBatchImportDTO.setTableName(deptName);
                    wbsTreeBatchImportDTO.setInitTableName(initTableName);
                    list.add(wbsTreeBatchImportDTO);
                }
            });
            //新增元素
            list.stream().forEach(i -> {
                Long elementTableId = i.getId();
                String tableName = i.getTableName();
                String initTableName = i.getInitTableName();
                final int[] keyNumb = {0};
                result2.stream().forEach(map->{
                    if (tableName.equals(map.get("表名"))) {
                        keyNumb[0]++;
                        WbsFormElement wbsFormElement = new WbsFormElement();
                        wbsFormElement.setEName(map.get("字段名"));
                        String key = "key_" + keyNumb[0];
                        wbsFormElement.setEKey(key);
                        wbsFormElement.setFId(String.valueOf(elementTableId));
                        wbsFormElement.setELength(getElementLength(map.get("数据类型")));
                        wbsFormElement.setEType(getElementType(map.get("数据类型")));
                        wbsFormElement.setEInspectionMethod(map.get("计算公式、方法或数值"));
                        wbsFormElement.setEAllowDeviation(map.get("允许偏差范围"));
                        wbsFormElement.setRemark(map.get("备注"));
                        wbsFormElementService.save(wbsFormElement);
                        //新增实体字段
                        baseMapper.alterTableFiled(initTableName,
                                wbsFormElement.getEKey(),
                                getInitTableFiledType(wbsFormElement.getEType()),
                                wbsFormElement.getELength());
                    }
                });
            });
            return true;
        } catch (
                IOException | ClassNotFoundException e) {
            e.printStackTrace();
        } finally {
            File file2 = new File(canonicalPath);
            if (file2.isFile() && file2.exists()) {
                file2.delete();
                System.gc();
            }
        }
        return false;
    }


    private Boolean initTable(String tableName) {
        //建表
        Integer row = baseMapper.createTable(tableName);
        if (row >= 0) {
            return true;
        }
        return false;
    }

    private String getInitTableFiledType(Integer type) {
        switch (type) {
            case 2:
                return "bigint"; //整数
            case 3:
                return "decimal"; //小数
            case 4:
                return "datetime"; //日期
            case 5:
                return "decimal";  //数值
            case 6:
                return "varchar"; //签名
            case 7:
                return "varchar"; //文件
            case 1:
            default:
                return "varchar"; //字符串
        }
    }

    private Integer getElementType(String type) {
        switch (type) {
            case "字符串":
                return 1;
            case "整数":
                return 2;
            case "小数":
                return 3;
            case "日期":
                return 4;
            case "数值":
                return 5;
            case "签名":
                return 6;
            case "文件":
                return 7;
            default:
                return 0;
        }
    }

    private Integer getElementLength(String type) {
        switch (type) {
            case "字符串":
            case "签名":
            case "文件":
                return 1000;
            case "整数":
                return 255;
            case "小数":
            case "数值":
                return 65;
            case "日期":
            default:
                return 0;
        }
    }

    private Integer getTableType(String tableType) {
        switch (tableType) {
            case "检验表":
                return 1;
            case "记录表":
                return 2;
            case "通用表":
                return 3;
            case "监表":
                return 4;
            case "评定表":
                return 5;
            case "测量表":
                return 6;
            case "竣工表":
                return 7;
            case "汇总表":
                return 8;
            case "试验记录表":
                return 9;
            case "试验报告表":
                return 10;
            default:
                return 0;
        }
    }

    @Override
    public List<WbsNodeTableVO> selectByNodeTable(String id) {
        //查询节点下的所有表单
        List<WbsNodeTableVO> wbsNodeTableVOS = wbsTreeMapper.selectByNodeTable(id);
        return wbsNodeTableVOS;
    }

    @Override
    public boolean removeTableById(String id) {
        Integer integer = wbsTreeMapper.removeTableById(id);
        if (integer > 0) {
            return true;
        }
        return false;
    }

    @Override
    public List<WbsFormElementVO> selectFormElements(String id) {
        List<WbsFormElementVO> wbsFormElements = wbsTreeMapper.selectFormElements(id);
        return wbsFormElements;
    }

    private String selectTableIdByTableName(String id) {
        QueryWrapper<WbsTree> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", id);
        WbsTree wbsTree = wbsTreeMapper.selectOne(queryWrapper);
        if (!("").equals(wbsTree) && wbsTree != null) {
            return "-1";

        }
        assert wbsTree != null;
        return String.valueOf(wbsTree.getId());
    }


    /**
     * wbs公有树节点excel导入
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean importWbsTree(MultipartFile excelFile, WbsTree wbsTreeFu, WbsTree wbsTree1) throws IOException {
        //赋值
        wbsTreeFu.setAncestors("0");
        wbsTree1.setProjectNodeId(wbsTree1.getId());
        File file = WbsExcelUtil.convert(excelFile);
        String canonicalPath = file.getCanonicalPath();
        try {
            //解析
            WbsExcelUtil excelUtil = new WbsExcelUtil();
            ArrayList<Map<String, String>> result = excelUtil.readExcelToObj(canonicalPath);
            return importExcel(result, wbsTreeFu, wbsTree1);
        } catch (Exception e) {
            String message = e.getMessage();
            throw new ServiceException("导入失败,message{}" + message);
        } finally {
            File file2 = new File(canonicalPath);
            if (file2.isFile() && file2.exists()) {
                file2.delete();
                System.gc();
            }
        }
    }

    private boolean importExcel(ArrayList<Map<String, String>> result, WbsTree wbsTreeFu, WbsTree wbsTree1) {
        result.stream().forEach(map -> {
            map.forEach((nodeType, nodeName) -> {
                WbsTree wbsTreeZi = new WbsTree();
                if (StringUtils.isNotEmpty(nodeName)) {
                    //初始化
                    wbsTreeZi.setTenantId(AuthUtil.getTenantId());
                    wbsTreeZi.setDeptName(nodeName);
                    wbsTreeZi.setDeptCategory(Integer.valueOf(nodeType));
                    wbsTreeZi.setWbsId(wbsTreeFu.getWbsId());
                    wbsTreeZi.setParentId(wbsTreeFu.getParentId());
                    wbsTreeZi.setAncestors(wbsTreeFu.getAncestors());
                    wbsTreeZi.setProjectNodeId(wbsTree1.getProjectNodeId());
                    //新增节点返回未创建成功查询出来的wbsTree对象、创建成功后的wbsTreeZi对象
                    WbsTree wbsTree = importTree(wbsTreeZi);
                    //后置初始化
                    if (wbsTree == null) {
                        Long id = wbsTreeZi.getId();
                        //初始化ancestors
                        StringBuilder stringBuffer = new StringBuilder();
                        StringBuilder appendStr = stringBuffer.append(",").append(wbsTreeZi.getParentId());
                        String ancestors = wbsTreeZi.getAncestors() + appendStr;
                        //修改ancestors
                        updateData(ancestors, id);
                        wbsTreeFu.setParentId(id);
                        wbsTreeFu.setAncestors(ancestors);
                    } else {
                        wbsTreeFu.setParentId(wbsTree.getId());
                        wbsTreeFu.setAncestors(wbsTree.getAncestors());
                    }
                }
            });
        });

        //修改ProjectNodeId
        int i = baseMapper.updateById(wbsTree1);
        return i > 0;
    }

    private WbsTree importTree(WbsTree wbsTreeZi) {
        QueryWrapper<WbsTree> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("wbs_id", wbsTreeZi.getWbsId());
        queryWrapper.eq("dept_name", wbsTreeZi.getDeptName());
        queryWrapper.eq("dept_category", wbsTreeZi.getDeptCategory());
        queryWrapper.eq("project_node_id", wbsTreeZi.getProjectNodeId());
        queryWrapper.ne("ancestors", wbsTreeZi.getAncestors());
        WbsTree wbsTrees = baseMapper.selectOne(queryWrapper);
        if (wbsTrees == null) {
            //初始化
            wbsTreeZi.setStatus(1);
            wbsTreeZi.setType(1);
            wbsTreeZi.setMajorDataType(0);
            if (wbsTreeZi.getDeptCategory() == 6) {
                wbsTreeZi.setIsConcrete(0);
                wbsTreeZi.setIsExpernode(0);
            }
            baseMapper.insert(wbsTreeZi);
        }
        //TODO 暂未处理重复条新增问题
        return wbsTrees;
    }

    @Override
    public int deleteBatchByProjectNodeId(Long projectNodeId) {
        return baseMapper.deleteBatchProjectNodeId(projectNodeId);
    }

    @Override
    public WbsTree selectGenNodeByWbsId(Long id) {
        return baseMapper.selectByWbsId(id);
    }

    @Override
    public List<WbsTree> findByWbsTreeNode(String ids) {
        return baseMapper.selectList(Wrappers.<WbsTree>query().lambda()
                .eq(WbsTree::getWbsId, ids)
                .ne(WbsTree::getParentId, 0)
                .eq(WbsTree::getStatus, 1)
                .eq(WbsTree::getIsDeleted, 0));
    }

    @Override
    public List<SaveUserInfoByProjectVO2> findProjectAndContractList(Long userId) {
        List<SaveUserInfoByProjectVO2> list = baseMapper.selectListByUserId(userId);
        return list;
    }

    @Override
    public List<WbsTreeDTO2> findWbsTreeSameLevel(String parentId) {
        List<WbsTreeDTO2> trees = baseMapper.selectWbsTreeListByParentId(parentId);
        return trees;
    }

    @Override
    public boolean wbsTreeSort(List<WbsTreeDTO2> wbsTreeList) {
        int number = 1;
        for (WbsTreeDTO2 wbsTree : wbsTreeList) {
            wbsTree.setSort(number);
            //修改sort
            baseMapper.updateSortById(wbsTree.getId(), wbsTree.getSort());
            number++;
        }
        return true;
    }

    @Override
    public int deleteLogicByWbsId(String ids) {
        return baseMapper.deleteLogicByWbsId(ids);
    }

    @Override
    public List<WbsTree> selectByWbsId(String id) {
        return baseMapper.selectList(Wrappers.<WbsTree>query().lambda()
                .eq(WbsTree::getParentId, id)
                .eq(WbsTree::getStatus, 1)
        );
    }

    private boolean updateData(String ancestors, Long id) {
        Integer row = baseMapper.updateById2(ancestors, id);
        if (row > 0) {
            return true;
        }
        return false;
    }


    /**
     * 项目级wbs树保存/修改
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String submitWbsTreeInProject(WbsTreeContractDTO pawDTO) {
        String wbsTreeIds = pawDTO.getWbsTreeIds();
        String[] ids = wbsTreeIds.split(",");
        List<String> idList = Arrays.asList(ids);
        List<String> idList1 = idList.stream().filter(string -> !string.isEmpty()).collect(Collectors.toList());
        List<String> idList2 = new ArrayList<>();
        int deletedStatus = 0;
        if (pawDTO.getReferenceType().equals("public")) {
            //当前公有引用
            List<WbsTreePrivate> wbsTreePrivates = wbsTreePrivateMapper.selectList(Wrappers.<WbsTreePrivate>query().lambda()
                    .eq(WbsTreePrivate::getWbsId, pawDTO.getWbsId())
                    .eq(WbsTreePrivate::getProjectId, pawDTO.getProjectId())
                    .eq(WbsTreePrivate::getType, 1)
            );
            wbsTreePrivates.stream().forEach(wbsTreePrivate -> {
                idList2.add(String.valueOf(wbsTreePrivate.getId()));

            });
        } else if (pawDTO.getReferenceType().equals("private")) {
            //私有引用
            List<WbsTreePrivate> wbsTreePrivates2 = wbsTreePrivateMapper.selectList(Wrappers.<WbsTreePrivate>query().lambda()
                    .eq(WbsTreePrivate::getWbsId, pawDTO.getPrimaryKeyId())
                    .eq(WbsTreePrivate::getProjectId, pawDTO.getProjectId())
                    .eq(WbsTreePrivate::getType, 1)
            );
            wbsTreePrivates2.stream().forEach(wbsTreePrivate -> {
                idList2.add(String.valueOf(wbsTreePrivate.getId()));
            });
            deletedStatus = 1;
        }
        List<String> diffrent = WbsTreeContractServiceImpl.getDiffrent(idList1, idList2);
        if (diffrent.size() == 0) {
            List<String> tableId1 = new ArrayList<>();
            List<String> tableId2 = new ArrayList<>();
            //公有树是否新增元素表，同步到私有
            List<WbsTree> allNodeList = findAllNodeList(wbsTreeIds);
            List<WbsTree> wbsTreeListAll = new ArrayList<>();
            allNodeList.stream().forEach(wbsTree -> {
                List<WbsTree> wbsTreeList = wbsTreeMapper.selectList(Wrappers.<WbsTree>query().lambda()
                        .eq(WbsTree::getWbsId, pawDTO.getWbsId())
                        .eq(WbsTree::getType, 2)
                        .eq(WbsTree::getParentId, wbsTree.getId())
                );
                wbsTreeListAll.addAll(wbsTreeList);
            });
            wbsTreeListAll.stream().forEach(wbsTree -> {
                tableId1.add(String.valueOf(wbsTree.getId()));
            });

            //获取项目私有树下所有的元素表
            List<WbsTreePrivate> wbsTreePrivates1 = wbsTreePrivateMapper.selectList(Wrappers.<WbsTreePrivate>query().lambda()
                    .eq(WbsTreePrivate::getWbsId, pawDTO.getWbsId())
                    .eq(WbsTreePrivate::getProjectId, pawDTO.getProjectId())
                    .eq(WbsTreePrivate::getType, 2)
            );
            wbsTreePrivates1.stream().forEach(wbsTreePrivate -> {
                tableId2.add(String.valueOf(wbsTreePrivate.getId()));
            });
            List<String> diffrent1 = WbsTreeContractServiceImpl.getDiffrent(tableId1, tableId2);
            if (diffrent1.size() == 0) {
                throw new ServiceException("未进行任何操作");
            } else {
                if (tableId1.size() > tableId2.size()) {
                    //同步元素表
                    diffrent1.stream().forEach(tableId -> {
                        WbsTree wbsTree = wbsTreeMapper.selectById(tableId);
                        Long snowId1 = SnowFlakeUtil.getId();
                        wbsTreePrivateMapper.insertCombination1(snowId1, wbsTree, pawDTO.getWbsType(), pawDTO.getProjectId());
                    });
                    return "1";
                }
                //同步删除 TODO
            }
        } else {
            //新增
            if (idList1.size() > idList2.size()) {
                List<WbsTreePrivate> insertData = new ArrayList<>();  //数据初始化节点-引用公有
                List<WbsTreePrivate> insertData2 = new ArrayList<>();  //数据初始化节点-引用私有
                if (pawDTO.getReferenceType().equals("public")) {
                    //公有
                    List<List<WbsTree>> wbsTrees = new ArrayList<>();
                    diffrent.stream().forEach(id -> {
                        //查询所有公共节点 表单
                        List<WbsTree> trees = baseMapper.selectNodeAndTable(Long.valueOf(id));
                        wbsTrees.add(trees);
                    });
                    wbsTrees.stream().forEach(trees -> {
                        trees.stream().forEach(tree -> {
                            WbsTreePrivate wbsTreePrivate2 = getWbsTreePrivate1(tree, pawDTO);
                            insertData.add(wbsTreePrivate2);
                        });
                    });
                    projectInfoMapper.updateTemplateIdById(pawDTO.getProjectId(), pawDTO.getWbsId(), "public");

                } else if (pawDTO.getReferenceType().equals("private")) {
                    //私有
                    List<List<WbsTreePrivate>> wbsTreesPrivate = new ArrayList<>();
                    diffrent.stream().forEach(id -> {
                        //查询所有私有节点 表单
                        List<WbsTreePrivate> wbsTreePrivate = wbsTreePrivateMapper.selectNodeAndTable(id, pawDTO.getWbsId(), pawDTO.getReferencePrivateWbsProjectId());
                        wbsTreesPrivate.add(wbsTreePrivate);
                    });
                    wbsTreesPrivate.stream().forEach(wbsTreePrivates -> {
                        wbsTreePrivates.stream().forEach(wbsTreePrivate -> {
                            WbsTreePrivate wbsTreePrivate2 = getWbsTreePrivate2(wbsTreePrivate, pawDTO);
                            insertData2.add(wbsTreePrivate2);
                        });
                    });
                    projectInfoMapper.updateTemplateIdById(pawDTO.getProjectId(), pawDTO.getPrimaryKeyId(), "private");
                }
                if (pawDTO.getReferenceType().equals("public")) {
                    wbsTreePrivateService.saveBatch(insertData, 10000);
                } else if (pawDTO.getReferenceType().equals("private")) {
                    wbsTreePrivateService.saveBatch(insertData2, 10000);
                }
            } else {
                //删除节点 表单
                ArrayList<Long> ids1 = new ArrayList<>();
                ArrayList<Long> ids2 = new ArrayList<>();
                diffrent.stream().forEach(id -> {
                    ids1.add(Long.valueOf(id));
                });
                if (deletedStatus == 1) {
                    diffrent.stream().forEach(id -> {
                        ids2.add(Long.valueOf(id));
                    });
                }
                if (pawDTO.getReferenceType().equals("public")) {
                    baseMapper.deleteBatch(ids1, pawDTO.getWbsId(), pawDTO.getProjectId());
                    baseMapper.deleteBatch2(ids1, pawDTO.getWbsId(), pawDTO.getProjectId());
                } else if (pawDTO.getReferenceType().equals("private")) {
                    baseMapper.deleteBatch(ids2, pawDTO.getPrimaryKeyId(), pawDTO.getProjectId());
                    baseMapper.deleteBatch2(ids2, pawDTO.getPrimaryKeyId(), pawDTO.getProjectId());
                }
            }
        }
        return "2";
    }

    public List<WbsTree> findAllNodeList(String wbsTreeIds) {
        String[] ids = wbsTreeIds.split(",");
        List<Long> idList = new ArrayList<>();
        for (String id : ids) {
            idList.add(Long.valueOf(id));
        }
        return baseMapper.selectBatchIds(idList);
    }

    private WbsTreePrivate getWbsTreePrivate1(WbsTree wbsTree, WbsTreeContractDTO pawDTO) {
        WbsTreePrivate wbsTreePrivate = new WbsTreePrivate();
        //数据初始化
        Long snowId = SnowFlakeUtil.getId();
        wbsTreePrivate.setPKeyId(snowId);
        wbsTreePrivate.setId(wbsTree.getId());
        wbsTreePrivate.setWbsId(wbsTree.getWbsId());
        wbsTreePrivate.setWbsType(String.valueOf(pawDTO.getWbsType()));
        wbsTreePrivate.setProjectId(pawDTO.getProjectId());
        wbsTreePrivate.setTenantId(wbsTree.getTenantId());
        wbsTreePrivate.setParentId(wbsTree.getParentId());
        wbsTreePrivate.setAncestors(wbsTree.getAncestors());
        wbsTreePrivate.setDeptCategory(wbsTree.getDeptCategory());
        wbsTreePrivate.setDeptName(wbsTree.getDeptName());
        wbsTreePrivate.setFullName(wbsTree.getFullName());
        wbsTreePrivate.setSort(wbsTree.getSort());
        wbsTreePrivate.setRemark(wbsTree.getRemark());
        wbsTreePrivate.setType(wbsTree.getType());
        wbsTreePrivate.setTableType(wbsTree.getTableType());
        wbsTreePrivate.setStatus(wbsTree.getStatus());
        wbsTreePrivate.setIsDeleted(wbsTree.getIsDeleted());
        wbsTreePrivate.setUniqueCode(wbsTree.getUniqueCode());
        wbsTreePrivate.setPartitionCode(wbsTree.getPartitionCode());
        wbsTreePrivate.setIsConcrete(wbsTree.getIsConcrete());
        wbsTreePrivate.setIsExpernode(wbsTree.getIsExpernode());
        wbsTreePrivate.setTableOwner(wbsTree.getTableOwner());
        wbsTreePrivate.setMajorDataType(wbsTree.getMajorDataType());
        wbsTreePrivate.setInitTableName(wbsTree.getInitTableName());
        wbsTreePrivate.setIsLinkTable(wbsTree.getIsLinkTable());
        wbsTreePrivate.setIsExistForm(wbsTree.getIsExistForm());
        return wbsTreePrivate;
    }

    private WbsTreePrivate getWbsTreePrivate2(WbsTreePrivate wbsTree, WbsTreeContractDTO pawDTO) {
        WbsTreePrivate wbsTreePrivate = new WbsTreePrivate();
        //数据初始化
        Long snowId = SnowFlakeUtil.getId();
        wbsTreePrivate.setPKeyId(snowId);
        wbsTreePrivate.setId(wbsTree.getId());
        wbsTreePrivate.setWbsId(pawDTO.getPrimaryKeyId());
        wbsTreePrivate.setWbsType(String.valueOf(pawDTO.getWbsType()));
        wbsTreePrivate.setProjectId(pawDTO.getProjectId());
        wbsTreePrivate.setTenantId(wbsTree.getTenantId());
        wbsTreePrivate.setParentId(wbsTree.getParentId());
        wbsTreePrivate.setAncestors(wbsTree.getAncestors());
        wbsTreePrivate.setDeptCategory(wbsTree.getDeptCategory());
        wbsTreePrivate.setDeptName(wbsTree.getDeptName());
        wbsTreePrivate.setFullName(wbsTree.getFullName());
        wbsTreePrivate.setSort(wbsTree.getSort());
        wbsTreePrivate.setRemark(wbsTree.getRemark());
        wbsTreePrivate.setType(wbsTree.getType());
        wbsTreePrivate.setTableType(wbsTree.getTableType());
        wbsTreePrivate.setStatus(wbsTree.getStatus());
        wbsTreePrivate.setIsDeleted(wbsTree.getIsDeleted());
        wbsTreePrivate.setUniqueCode(wbsTree.getUniqueCode());
        wbsTreePrivate.setPartitionCode(wbsTree.getPartitionCode());
        wbsTreePrivate.setIsConcrete(wbsTree.getIsConcrete());
        wbsTreePrivate.setIsExpernode(wbsTree.getIsExpernode());
        wbsTreePrivate.setTableOwner(wbsTree.getTableOwner());
        wbsTreePrivate.setMajorDataType(wbsTree.getMajorDataType());
        wbsTreePrivate.setInitTableName(wbsTree.getInitTableName());
        wbsTreePrivate.setIsLinkTable(wbsTree.getIsLinkTable());
        wbsTreePrivate.setIsExistForm(wbsTree.getIsExistForm());
        return wbsTreePrivate;
    }

    @Override
    public WbsTreeAllListVO findWbsTreeList(Integer type) {
        WbsTreeAllListVO wbsTreeAllListVO = new WbsTreeAllListVO();
        //Wbs总树
        List<WbsInfo> wbsInfos = wbsInfoMapper.selectAll(type);
        wbsTreeAllListVO.setWbsInfos(wbsInfos);
        //项目私有树
        List<WbsTreePrivate> wbsTreePrivates = wbsTreePrivateMapper.selectAll(type);
        wbsTreeAllListVO.setWbsTreePrivates(wbsTreePrivates);
        return wbsTreeAllListVO;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public R saveFormElement(FormElementDTO formElementDTO) {
        if (formElementDTO.getElementList().size() <= 0) {
            return R.fail("操作失败,请先添加表单元素");
        }
        String filedName = "";
        String filedType = "";
        String filedLength = "";
        for (WbsFormElement wbsFormElement : formElementDTO.getElementList()) {
            filedName = getPinyin(wbsFormElement.getEName(), "");
            filedType = String.valueOf(wbsFormElement.getEType());
            filedLength = String.valueOf(wbsFormElement.getELength());
        }
        if (StringUtils.isEmpty(filedName) || StringUtils.isEmpty(filedType) || StringUtils.isEmpty(filedLength)) {
            return R.fail("操作失败,请完整填写元素参数");
        }

        String deptName = formElementDTO.getDeptName();
        if (deptName.length() > 100 || deptName.length() < 1) {
            throw new ServiceException("表名长度错误,输入范围1-100个字符长度");
        }
        //初始化表名
        //String realName = getFirstSpell(deptName);
        Long id = SnowFlakeUtil.getId();
        String newTableName = "m_wbs_form_" + DateUtil.time() + "_" + id.toString();
        formElementDTO.setInitTableName(newTableName);

        //创建元素表
        boolean b1 = submit2(formElementDTO);

        List<WbsFormElement> elementList = formElementDTO.getElementList();
        int i = 1;
        for (WbsFormElement wbsFormElement : elementList) {
            if (wbsFormElement.getEType() == 1 && (wbsFormElement.getELength() > 1000 || wbsFormElement.getELength() < 10)) {
                throw new ServiceException("请输入正确的字符串长度,范围为10-1000");
            }
            if (wbsFormElement.getEType() == 2 && (wbsFormElement.getELength() > 255 || wbsFormElement.getELength() < 10)) {
                throw new ServiceException("请输入正确的整数长度,范围为10-255");
            }
            if (wbsFormElement.getEType() == 3 && (wbsFormElement.getELength() > 65 || wbsFormElement.getELength() < 10)) {
                throw new ServiceException("请输入正确的小数长度,范围为10-65");
            }
            //当前元素表中元素对应实体表唯一key值、fId
            wbsFormElement.setEKey("key_" + i++);
            wbsFormElement.setFId(String.valueOf(formElementDTO.getId()));
        }
        //新增元素
        boolean b2 = wbsFormElementService.saveBatch(elementList);

        //找到当前元素表中所有元素
        List<WbsFormElement> list = wbsFormElementService.selectElementListByFid(String.valueOf(formElementDTO.getId()));
        //初始化实体表
        Boolean b3 = wbsFormElementService.initTable(list, newTableName);
        if (b1 && b2 && b3) {
            return R.data(formElementDTO, "新增表单、元素、初始化实体表成功");
        } else {
            throw new ServiceException("操作失败");
        }
    }

    public static String getFirstSpell(String chinese) {
        StringBuffer pybf = new StringBuffer();
        char[] arr = chinese.toCharArray();
        HanyuPinyinOutputFormat defaultFormat = new HanyuPinyinOutputFormat();
        defaultFormat.setCaseType(HanyuPinyinCaseType.LOWERCASE);
        defaultFormat.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
        for (int i = 0; i < arr.length; i++) {
            if (arr[i] > 128) {
                try {
                    String[] temp = PinyinHelper.toHanyuPinyinStringArray(arr[i], defaultFormat);
                    if (temp != null) {
                        pybf.append(temp[0].charAt(0));
                    }
                } catch (BadHanyuPinyinOutputFormatCombination e) {
                    e.printStackTrace();
                }
            } else {
                pybf.append(arr[i]);
            }
        }
        return pybf.toString().replaceAll("\\W", "").trim();
    }

    public static String getPinyin(String text, String separator) {
        //text 文本, separator 转换后添加的分隔符
        char[] chars = text.toCharArray();
        HanyuPinyinOutputFormat format = new HanyuPinyinOutputFormat();
        // 设置大小写
        format.setCaseType(HanyuPinyinCaseType.LOWERCASE);
        // 设置声调表示方法
        format.setToneType(HanyuPinyinToneType.WITHOUT_TONE);
        // 设置字母u表示方法
        format.setVCharType(HanyuPinyinVCharType.WITH_V);
        String[] s;
        String rs = "";
        try {
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < chars.length; i++) {
                // 判断是否为汉字字符
                if (String.valueOf(chars[i]).matches("[\\u4E00-\\u9FA5]+")) {
                    s = PinyinHelper.toHanyuPinyinStringArray(chars[i], format);
                    if (s != null) {
                        sb.append(s[0]).append(separator);
                        continue;
                    }
                }
                sb.append(chars[i]);
                if ((i + 1 >= chars.length) || String.valueOf(chars[i + 1]).matches("[\\u4E00-\\u9FA5]+")) {
                    sb.append(separator);
                }
            }
            rs = sb.substring(0, sb.length());
        } catch (BadHanyuPinyinOutputFormatCombination e) {
            e.printStackTrace();
        }
        return rs;
    }

    public int updateByCondition(Long id, String wbsName, Integer status) {
        int row = baseMapper.updateByCondition(id, wbsName, status);
        return row;
    }
}
